#include <stdbool.h>

#include "keyboard.h"
#include "kprintf.h"
#include "sys/io.h"
#include "terminal.h"

uint8_t terminal_getkey(void)
{
	static bool caps_mode = false;
	static uint8_t prev_scan_code = 0;
	uint8_t scan_code;

	scan_code = inb(KEYBOARD_PORT);
	if (scan_code == 0x3A || scan_code == 0x58) {
		caps_mode = !caps_mode;
		outb(0xED, (caps_mode << 2)); // turn on/off capslock led
	}
	else if (scan_code == KEY_RIGHT_SHIFT || scan_code == KET_LEFT_SHIFT)
		caps_mode = true;
	else if (scan_code == KEY_RIGHT_SHIFT + 128 || scan_code == KET_LEFT_SHIFT + 128)
		caps_mode = false;
	else if (scan_code != prev_scan_code && prev_scan_code != 0) {
		if (scan_code < 128 && keymap[scan_code])
			kprintf(0, "%c", keymap[scan_code][caps_mode]);
		if (scan_code >= KEY_F1 && scan_code <= KEY_F10)
			terminal_set_screen(scan_code - KEY_F1);
	}
	prev_scan_code = scan_code;
	return scan_code;
}
